/*************************************************
**Authors: Kevin Cartrette, Alex Lemke			**
**Co-Author: Tom Carbone						**
**Date:	11/30/2011								**
**Version:	1.0									**
**Summary:  Enemy class with modifed render and **
**update to branch from standard enemy behavior.**
*************************************************/

#include "BlackHoleEnemy.h"
#include <Windows.h>
#include <stdio.h>
#include <stdlib.h>
#include "stateManager.h"
#include "gl\glu.h"	
#include "ShapeDraw.h"
#include <time.h>
#include "SpriteManager.h"
#include "EnemyManager.h"
#include "enemy.h"
#include "FieldManager.h"
#include "field.h"
#include "PlayerManager.h"
#include "Player.h"
#include "ProtonEnemy.h"
#include "Random.h"
#include "gamePlayVariables.h"

void BlackHoleEnemyC::update(double64_t milliseconds)
{
	if(mEnabled){
		switch(StateManagerC::GetInstance()->getState())
		{
			case StateManagerC::GAMEPLAY:
				Vector3C *playerPosition = getPlayerPosition();

				FieldManagerC::GetInstance()->applyColor(&mPosition, 10, &enemyColor);

				//Velocity Change
				Vector3C velocityChange;
				velocityChange.x = playerPosition->x - mPosition.x;
				velocityChange.y = playerPosition->y - mPosition.y;
				velocityChange.normalize();
				velocityChange.scaleVector(0.0015f);
				mVelocity.add2DQuickVector(velocityChange);

				//Friction
				mVelocity.scaleVector(0.8f);

				//Increase Position
				mPosition.x += mVelocity.x * milliseconds;
				mPosition.y += mVelocity.y * milliseconds;

				if(strength > 0){
					//Pulling Other Enemies Towards it
					EnemyC* temporaryEnemy;
					for(int32_t i = 0; i <EnemyManagerC::GetInstance()->getNumEnemies(); i++)
					{
						temporaryEnemy = EnemyManagerC::GetInstance()->getEnemyPtr(i);
						Vector3C temporaryEnemyPositon = *(temporaryEnemy->getPosition());

						//Distance to the gravity well
						float_t distance = mPosition.distance(temporaryEnemyPositon);
						if(distance != 0){
							if((distance < 500.0f) || (distance < (strength * 10.0f)))
							{
								if(distance < 50.0f){

									switch(StateManagerC::GetInstance()->getGameMode())
									{
										case StateManagerC::WORM_HOLE_MODE:
											strength++;
											if(strength >= 10)
												strength = 49;
											break;
										default:
											strength++;
											EnemyManagerC::GetInstance()->getEnemyPtr(i)->disable();
											break;
									}

									if(strength >= 50){ // The gravity well has exploded!
										disable();
										//Create the proton enemies!
										EnemyManagerC::GetInstance()->createProtons(mPosition);

										//Push the grid
										Vector3C enemyPullColor = enemyColor;
										enemyPullColor.scaleVector(0.25f);
										FieldManagerC::GetInstance()->applyPush(&mPosition, (strength * 10.0f), (-0.015f), &enemyPullColor);

										break;
									}
								}
								
								//Direction towards the gravity well!
								Vector3C pullDirection;
								pullDirection.x = mPosition.x - temporaryEnemyPositon.x;
								pullDirection.y = mPosition.y - temporaryEnemyPositon.y;

								//Pull direction towards gravity well
								pullDirection.normalize();//(0.00175f)
								//pullDirection.scaleVector((0.00575f) * (501.0f - distance) * (strength/50.0f));
								pullDirection.scaleVector((1/(distance/2)) * (strength * BLACK_HOLE_STRENGTH));

								temporaryEnemy->applyForce(pullDirection);
							}
						}
					}

					//Pulling Players towards it
					PlayerC* temporaryPlayer;
					for(int32_t i = 0; i <PlayerManagerC::GetInstance()->getNumPlayers(); i++)
					{
						temporaryPlayer = PlayerManagerC::GetInstance()->getPlayerPtr(i);
						Vector3C temporaryPlayerPositon = *(temporaryPlayer->getPosition());

						float_t distance = mPosition.distance(temporaryPlayerPositon);
						if((distance < 500.0f) || (distance < (strength * 10.0f)))
						{
							Vector3C pullDirection;
							pullDirection.x = mPosition.x - temporaryPlayerPositon.x;
							pullDirection.y = mPosition.y - temporaryPlayerPositon.y;

							pullDirection.normalize();
							//pullDirection.scaleVector((0.00175f) * (501.0f - distance) * (strength/50.0f));
							pullDirection.scaleVector((1/(distance/2)) * (strength * BLACK_HOLE_STRENGTH));
							temporaryPlayer->applyForce(pullDirection);
						}
					}

					//Pulling the grid towards it!
					FieldManagerC::GetInstance()->applyGravityPull(&mPosition, 300 * (strength / 25.0f), ((strength / 25.0f) * 0.015f), &enemyColor);//(pullAmount * 0.1f));
				}

#if(ENEMY_TEXTURES)
				enemySprite.setPosition(mPosition.x, mPosition.y, 0);
#endif
				DoCollisions();
				break;
		}
	}
}

void BlackHoleEnemyC::init(float_t initPosX, float_t initPosY, float_t initVelX, float_t initVelY,int32_t id)
{
	mPosition.x = initPosX;
	mPosition.y = initPosY;
	mVelocity.x = initVelX;
	mVelocity.y = initVelY;

	mVelocity.x = 0;
	mVelocity.y = 0;

	mRadius = ENEMY_RADIUS;
	scoreValue = BLACK_HOLE_SCORE;
	health = 10;

	strength = 0;

	enemyColor.x = 1.0f;
	enemyColor.y = 0.5f;
	enemyColor.z = 0.0f;

#if(ENEMY_TEXTURES)
	enemySprite.setUp(9,0,0,0,64 * 2,64 * 2,0,0,0,64,64);
	enemySprite.setCentered(true);
#endif
}

void BlackHoleEnemyC::render()
{
	if(mEnabled){
		glPushMatrix();
#if(ENEMY_TEXTURES)
		enemySprite.setRotation(enemySprite.returnRotation() + (0.1f * strength));
		//DrawCircle(30.0f, mPosition.x,mPosition.y,1.0f,0.0f,0.0f,false);
		SpriteManagerC::GetInstance()->drawSprite(enemySprite);
#else
		char8_t red = (char8_t)((mEnemyColor>>16)& 0xFF);
		char8_t green = (char8_t)((mEnemyColor>>8) & 0xFF);
		char8_t blue = (char8_t)((mEnemyColor>>0) & 0xFF);
		bool filledVal;

		filledVal = true;

		DrawCircle(mRadius, mPosition.x,mPosition.y,red,green,blue,filledVal);
#endif
		//Pop Matrix
		glPopMatrix();

#if DRAW_COLLISIONS == 1
	DrawCircle(mRadius, mPosition.x, mPosition.y, 1.0f, 0.0f, 0.0f, false);
#endif
	}
}

void BlackHoleEnemyC::decreaseHealth()
{
	health--;
	strength += 5;
}