/*************************************************
**Authors: Kevin Cartrette, Alex Lemke			**
**Co-Author: Tom Carbone						**
**Date:	11/30/2011								**
**Version:	1.0									**
**Summary:  Basic particle system manager deals **
**with the processing of all particles in game. **
*************************************************/

#include <windows.h>										
#include <stdio.h>												
#include <stdarg.h>												
#include <math.h>											
#include "gl\gl.h"												
#include "gl\glu.h"												
#include "gl\glut.h"
#include "baseTypes.h"
#include "Vector3.h"
#include "Sprite.h"
#include "Particle.h"
#include "random.h"
#include "FieldManager.h"
#include "field.h"
#include "random.h"
#include "CameraManager.h"
#include "SpriteManager.h"
#include "soil.h"
#include "gamePlayVariables.h"

ParticleC::ParticleC()
{
}

ParticleC::~ParticleC()
{
}

void ParticleC::init(uint32_t initTextureId, Vector3C initPosition, Vector3C initVelocity, Vector3C initAngularVelocity, Vector3C initColor, float_t initSize, float_t initTimeToLive)
{
	//Init all variables.
	textureId = initTextureId;
	position.setEqualToVector(initPosition);
	velocity.setEqualToVector(initVelocity);
	color.setEqualToVector(initColor);
	angularVelocity.setEqualToVector(initAngularVelocity);
	size = initSize;
	timeToLive = initTimeToLive;
	currentTime = 0;
	alive = true;
	randomColor = false;
	bounceOffWalls = false;
#if(PARTICLE_TEXTURES)
	particleSprite.setUp(textureId,0,0,0,4 * 2,4 * 2,0,0,0,4,4);
	particleSprite.setCentered(true);
#endif
}

bool ParticleC::update(double64_t milliseconds)
{
	//While particle is alive, increase current time and velocity.
	if(alive == true)
	{
		currentTime += milliseconds;
		position.x += velocity.x*milliseconds/getRangedRandom(PARTICLE_RANDOM_VELOCITY_MIN_MODIFER,PARTICLE_RANDOM_VELOCITY_MAX_MODIFER);
		position.y += velocity.y*milliseconds/getRangedRandom(PARTICLE_RANDOM_VELOCITY_MIN_MODIFER,PARTICLE_RANDOM_VELOCITY_MAX_MODIFER);

		//position.x += velocity.x*milliseconds;
		//position.y += velocity.y*milliseconds;
		//Apply Friction
		velocity.scaleVector(PARTICLE_FRICTION);

		//velocity.x *= velocity.x
#if(PARTICLE_USE_TEXTURES)
		particleSprite.setPosition(position.x, position.y, position.z);
#endif
		//If currentTime is greater then time to live, particle is died.
		if(currentTime >= timeToLive)
		{
			alive = false;
			return false;
		}

#if(PARTICLE_BOUNCE_OFF_WALL)
		//If particle should bounce off wall, collide with game field.
		if(bounceOffWalls == true)
		{
			collideField();
		}
#endif
		return true;
	}
	return true;
}

void ParticleC::draw()
{
	//If particle is alive, draw.
	if(alive == true)
	{
		if(CameraManagerC::GetInstance()->checkIfInViewport(position)){ //Only draw if it's in the current camera view
			//Push Matrix
			glPushMatrix();
#if(PARTICLE_USE_TEXTURES)
			SpriteManagerC::GetInstance()->drawParticle(particleSprite,getRangedRandom(0.0f,1.0f),getRangedRandom(0.0f,1.0f),getRangedRandom(0.0f,1.0f),0.5);
#else
			glColor3f(color.x,color.y,color.z);
			glLineWidth(size);
			glEnable(GL_LINE_SMOOTH);
			glBegin(GL_LINES);
			glVertex2f(position.x, position.y);
			glVertex2f(position.x + cos(angularVelocity.x) * PARTICLE_LENGTH_MODIFIER * (currentTime/timeToLive), (position.y + sin(angularVelocity.x) * PARTICLE_LENGTH_MODIFIER *(currentTime/timeToLive)));
			glEnd();
#endif
			//Pop Matrix
			glPopMatrix();
		}
	}
}

void ParticleC::collideField()
{
	FieldC *field = FieldManagerC::GetInstance()->getFieldPtr();
	float_t rightSide = field->getPosition()->x+((float_t)field->getWidth()/TWO);
	float_t leftSide = field->getPosition()->x-((float_t)field->getWidth()/TWO);
	float_t topSide = field->getPosition()->y+((float_t)field->getHeight()/TWO);
	float_t bottomSide = field->getPosition()->y-((float_t)field->getHeight()/TWO);
	
	//Box collision
	if(position.x <= leftSide  )
	{
		velocity.x *= -1;
		position.x  = leftSide;
	}
	if(position.x >= rightSide  )
	{
		velocity.x *= -1;
		position.x  = rightSide;
	}
	if(position.y <= bottomSide)
	{
		velocity.y *= -1;
		position.y = bottomSide;
	}
	if(position.y>= topSide  )
	{
		velocity.y *= -1;
		position.y = topSide;
	}
}

void ParticleC::shutDown()
{
	delete this;
}

bool ParticleC::isAlive()
{
	return alive;
}

void ParticleC::setRandomColor(bool newRandomColor)
{
	randomColor = newRandomColor;
	if(randomColor == true)
	{
		color.x = getRangedRandom(PARTICLE_RANDOM_COLOR_MIN_RED,PARTICLE_RANDOM_COLOR_MAX_RED);
		color.y = getRangedRandom(PARTICLE_RANDOM_COLOR_MIN_BLUE,PARTICLE_RANDOM_COLOR_MAX_BLUE );
		color.z = getRangedRandom(PARTICLE_RANDOM_COLOR_MIN_GREEN,PARTICLE_RANDOM_COLOR_MAX_GREEN);
	}
}

void ParticleC::setBounceOffWalls(bool newBounceOffWalls)
{
	bounceOffWalls = newBounceOffWalls;
}

bool ParticleC::getBounceOffWalls()
{
	return bounceOffWalls;
}

bool ParticleC::isRandomColor()
{
	return randomColor;
}

void ParticleC::setAlive(bool newAlive)
{
	alive = newAlive;
}