/*************************************************
**Authors: Kevin Cartrette, Alex Lemke			**
**Co-Author: Tom Carbone						**
**Date:	11/30/2011								**
**Version:	1.0									**
**Summary:  Basic particle system manager deals **
**with the processing of all particles in game. **
*************************************************/

#include "ParticleManager.h"
#include "time.h"
#include "math.h"
#include "Vector2.h"
#include "Vector3.h"
#include "stateManager.h"
#include "InputManager.h"
#include "PlayerManager.h"
#include "FieldManager.h"
#include "random.h"
#include "gamePlayVariables.h"

ParticleManagerC* ParticleManagerC::pInstance = NULL;

ParticleManagerC *ParticleManagerC::CreateInstance()
{
	if(pInstance!=NULL)return pInstance;
	else
	pInstance = new ParticleManagerC();
	return pInstance;
}

void ParticleManagerC::init()
{	
	//Seed random number generator.
	srand ((unsigned long)time(NULL) );
	Vector3C position(100,100,100);
	Vector3C velocity(0,0,0);
	Vector3C angularVelocity(0,0,0);


	//Getting a random color
	temp.x = getRangedRandom(PARTICLE_RANDOM_COLOR_MIN_RED,PARTICLE_RANDOM_COLOR_MAX_RED);
	temp.y = getRangedRandom(PARTICLE_RANDOM_COLOR_MIN_GREEN,PARTICLE_RANDOM_COLOR_MAX_GREEN);
	temp.z = getRangedRandom(PARTICLE_RANDOM_COLOR_MIN_BLUE,PARTICLE_RANDOM_COLOR_MAX_BLUE);

	//Ship emitter, special case. Location zero.
	int32_t shipEmitter = newEmitter(PointEmitter,SHIP_POINT_EMITTER_TIME_UPDATE,SHIP_POINT_EMITTER_TIME_TO_LIVE,position,velocity,angularVelocity,
		SHIP_POINT_EMITTER_SIZE,SHIP_POINT_EMITTER_MIN_PARTICLES,SHIP_POINT_EMITTER_MAX_PARTICLES, &temp);
	setTimeBased(shipEmitter,false);

	currentTime = 0;
	timeBetweenUpdates = PARTICLE_MANAGER_TIME_BETWEEN_UPDATES;
	timeSinceLastMenuUpdate = 0;
	timeBetweenMenuUpdates = PARTICLE_MANAGER_TIME_BETWEEN_MENU_UPDATES;
}

void ParticleManagerC::updateParticles(double64_t milliseconds)
{
	switch(StateManagerC::GetInstance()->getState())
	{
		//Emitt particles on the main menu screen.
		case StateManagerC::MAIN_MENU: 
		case StateManagerC::SCORE_MENU:
			currentTime += milliseconds;
			if(currentTime >= timeBetweenMenuUpdates)
			{
				timeSinceLastMenuUpdate += currentTime;

				if(timeSinceLastMenuUpdate >= timeBetweenMenuUpdates)
				{
					Vector3C position(getRangedRandom(-500.0f,500.0f), getRangedRandom(-500.0f,500.0f), 0.0f);
					Vector3C positionForce;
					positionForce.x = position.x;
					positionForce.y = position.y;
					Vector3C velocity(CIRCLE_EMITTER_VELOCITY,CIRCLE_EMITTER_VELOCITY,0);
					Vector3C angularVelocity(CIRCLE_EMITTER_ANGULAR_VELOCITY,CIRCLE_EMITTER_ANGULAR_VELOCITY,0);


					//Getting a random color
					temp.x = getRangedRandom(PARTICLE_RANDOM_COLOR_MIN_RED,PARTICLE_RANDOM_COLOR_MAX_RED);
					temp.y = getRangedRandom(PARTICLE_RANDOM_COLOR_MIN_GREEN,PARTICLE_RANDOM_COLOR_MAX_GREEN);
					temp.z = getRangedRandom(PARTICLE_RANDOM_COLOR_MIN_BLUE,PARTICLE_RANDOM_COLOR_MAX_BLUE);

					//Fix emitter working here.
					newEmitter(CircleEmitter,CIRCLE_EMITTER_TIME_UPDATE,CIRCLE_EMITTER_TIME_TO_LIVE,position,velocity,angularVelocity,1.1f,CIRCLE_EMITTER_MIN_PARTICLES,CIRCLE_EMITTER_MAX_PARTICLES, &temp);

					//Apply a force to the grid
					FieldManagerC::GetInstance()->applyPush(&positionForce, getRangedRandom(-100.0f,400.0f), getRangedRandom(-0.02f, 0.1f), &temp);
					timeSinceLastMenuUpdate = 0;
				}

				updateParticleEmitters(currentTime);
				currentTime = 0;
			}
			break;
		case StateManagerC::GAMEPLAY:
			currentTime += milliseconds;

			if(currentTime >= timeBetweenUpdates)
			{
				//Creating the particle trail behind the player's ship.
				Vector3C* playerVelocity = (PlayerManagerC::GetInstance()->getPlayerPtr(0))->getVelocity();
				Vector3C* playerPosition = (PlayerManagerC::GetInstance()->getPlayerPtr(0))->getPosition();
				if(playerVelocity->x != 0 || playerVelocity->y != 0)
				{
					particleEmitterVector[0]->setEmit(true);
					particleEmitterVector[0]->setActiveStatus(true);
					particleEmitterVector[0]->setPosition(*playerPosition);
					particleEmitterVector[0]->update(currentTime);
				}
				else
				{
					particleEmitterVector[0]->setEmit(false);
					particleEmitterVector[0]->update(currentTime);
				}

				updateParticleEmitters(currentTime);
				currentTime = 0;
			}
			break;
	}
}

void ParticleManagerC::updateParticleEmitters(double64_t currentTime)
{
	for ((particleEmitterIterator = particleEmitterVector.begin())++; particleEmitterIterator != particleEmitterVector.end();)
	{
		(*particleEmitterIterator)->update(currentTime);

		//If the particle emitter is not active then delete.
		if(!(*particleEmitterIterator)->isActive())
		{
			(*particleEmitterIterator)->shutdown();

			delete (*particleEmitterIterator);
			particleEmitterIterator = particleEmitterVector.erase(particleEmitterIterator);
		}
		else
		{
				particleEmitterIterator++;
		}
	}
}

int32_t ParticleManagerC::newEmitter(Emitters emitter, double64_t initTimeBetweenUpdates, double64_t initTotalTimeToEmit, Vector3C initPosition, Vector3C initVelocity, Vector3C initAngularVelocity,
		float_t initSize, int32_t initMinParticles, int32_t initMaxParticles, Vector3C* color)
{
	//Create emitter and init.
	if(emitter == PointEmitter)
	{
		particleEmitterVector.push_back(new ParticlePointEmitterC);
		particleEmitterVector[particleEmitterVector.size()-1]->setColor(color);
	}
	else if(emitter == CircleEmitter)
	{
		particleEmitterVector.push_back(new ParticleCircleEmitterC);
		particleEmitterVector[particleEmitterVector.size()-1]->setColor(color);
	}

	int32_t emitterSpot = particleEmitterVector.size() - 1;
	particleEmitterVector[emitterSpot]->init(initTimeBetweenUpdates, initTotalTimeToEmit, initPosition, initVelocity, initAngularVelocity,
		initSize, initMinParticles, initMaxParticles);
	//Return position of new emitter.
	return emitterSpot;
}

void ParticleManagerC::renderParticles()
{
	currentParticleCount = 0;
	for (particleEmitterIterator = particleEmitterVector.begin(); particleEmitterIterator != particleEmitterVector.end(); particleEmitterIterator++)
	{
		//Early out if hit max number of particles to render.
		currentParticleCount += (*particleEmitterIterator)->getNumberParticles();
		if(currentParticleCount >= MAX_PARTICLES_TO_RENDER)
		{
			break;
		}
		(*particleEmitterIterator)->renderParticles();
    }
}

void ParticleManagerC::setParticleBounceOffWalls(int32_t indexEmitter, bool newParticleBounceOffWalls)
{
	particleEmitterVector[indexEmitter]->setParticleBounceOffWalls(newParticleBounceOffWalls);
}

void ParticleManagerC::setTimeBased(int32_t indexEmitter, bool newTimeBased)
{
	particleEmitterVector[indexEmitter]->setTimeBased(newTimeBased);
}

void ParticleManagerC::shutdown()
{
	for (particleEmitterIterator = particleEmitterVector.begin(); particleEmitterIterator != particleEmitterVector.end();)
	{
		delete(*particleEmitterIterator);
		particleEmitterIterator = particleEmitterVector.erase(particleEmitterIterator);
	}
}

int32_t ParticleManagerC::getTotalNumberOfParticles()
{
	return(currentParticleCount);
}